;; filename:	Renard_1825_20130615.asm
;; Copyright (c) 2006,2007,2009 Phil Short
;;
;;  This program is provided free for you to use in any way that you wish,
;;  subject to the laws and regulations where you are using it.  Due diligence
;;  is strongly suggested before using this code.
;;
;;  The Author makes no warranty of any kind, express or implied, with regard
;;  to this program or the documentation contained in this document.  The
;;  Author shall not be liable in any event for incidental or consequential
;;  damages in connection with, or arising out of, the furnishing, performance
;;  or use of these programs.
;;
;;  --------------------------------------------------------------------------
;;  NOTE:
;;  This code is configured for regular use (the firmware generates 30 uS (at
;;  60 Hz) pulses to drive the SSRs, synchronized to the zero-crossing input
;;  pin).  Here are the DEFINE statements to change for PWM use (generates a
;;  variable width pulse sync'd to the zero-crossing signal) or DC use
;;  generates a variable-width pulse, not sync'd to anything).

;; PWM_build can be either 0 or 1
;#define PWM_build 1
;; DC_build can be either 0 or 1 (it should be 0 if PWM_build is 0)
;#define DC_build 1
    
 ;;remove bank location warnings
 ERRORLEVEL -302

 #define CLOCKRATE 32000000
 #define BAUDRATE 57600

;; ADDR_OFFSET should normally be 1.  It is changed to another value if you
;; wish for the particular PIC to ignore a certain number of (8-byte) records
;; at the start of a frame (for wireless operation, mostly).
	
 #define START_ADDR 1

;; CTR_LOCKOUT should be 1 for the DC build, can be a somewhat larger number
;; for non-PWM or non-DC builds.

 #define CTR_LOCKOUT 1

;; For positive_true outputs, remove the following define (normally, this
;; should be left in place).
;; #define OUTPUT_NEGATIVE_TRUE
	
;;  --------------------------------------------------------------------------
;;
;;  13 May, 2006  PJS First version sent out for review, discussion (thought
;;  to be working version.
;;
;;  20 May, 2006  PJS Modifications for new protocol, shorter timer interval,
;;  some code optimization on ISR.  Fixed bug in the code for decoding ESC
;;  sequences.
;;
;;  19 June, 2006 PJS Modified uart access sequence (changed order of reading
;;  the RCSTA and RXREG registers, made sure to clear rx errors).
;;  Moved this history note to a slightly lower location in the file.
;;
;;  1 July 2006 PJS Fixed bug relating to pass-through function.
;;
;;  8 Sept 2006 PJS Tri-state RA5 (in case there is an oscillator connected,
;;  even though this particular code uses the internal oscillator).
;;
;;  23 Oct 2006 PJS fixed silly bug introduced in last version.  This bug would
;;  cause RA0-RA2 and RA3 to remain tri-stated, preventing these first four
;;  outputs from working.
;;
;;  07 Apr 2007 PJS backtracking from later version - parameterized baud rate,
;;  line frequency, etc (i.e. added #defines).
;;  07 Apr 2007 PJS changed from itnernal to external oscillator, and changed
;;  baudrate to 57600.
;;  08 Apr 2007 PJS folded in the lockout code (still set up for 60 hz only)
;;  08 Apr 2007 PJS added line-frequency detection code (hopefully more robust
;;  in the presence of noise than the last attempt).
;;
;;  11 Jul 2007 PJS folded in the PWM and DC-only code, using build-time
;;  defines.  Among other things, this involved re-arranging the code and
;;  renaming the ctr_lockout variable.
;;
;;  11 Aug 2007 PJS made changes to allow zero-crossing SSRs to turn on with
;;  brightness of 255 (basically this change turns the output on all of the
;;  time if the brightness is 255).
;;
;;  7 Sept 2007 PJS added option for positive-true (source) outputs.
;;
;;  29 Dec 2007 PJS fixed bug that left all lights on at power-up.
;;
;;  15 Sep 2009 PJS added code that allows the controller to have an address
;;    offset assigned.  This address is relative to the prior PIC (controller)
;;    in the daisy-chain.
;;
;;  June 2013 PJS Added conditional assembly directives to allow usage with
;;    PIC16F1825, including operation at 32 Mhz with that processor.
;;
;;  Hardware mappings
;;  PIN 13 (RA0) - triac driver 1 (output)
;;  PIN 12 (RA1) - triac driver 2 (output)
;;  PIN 11 (RA2) - triac driver 3 (output)
;;  PIN 4  (RA3) - zero-crossing (input only)
;;  PIN 3  (RA4) - triac driver 0 (output)
;;  PIN 10 (RC0) - triac driver 4 (output)
;;  PIN 9  (RC1) - triac driver 5 (output)
;;  PIN 8  (RC2) - triac driver 6 (output)
;;  PIN 7  (RC3) - triac driver 7 (output)
;;  PIN 6  (RC4/TX) - uart_out (output)
;;  PIN 5  (RC5/RX) - uart_in (input)
;;  PIN 2  (RA5) - clock input
;;
;;  Weak pullups are used on pin RA5 (currently a spare pin).
;;  Outputs are good to drive 15 mA opto-isolators.
;;
;;  This version of the code does not have any fast-path through the ISR, has
;;  good worst-case ISR path length (but mediocre average-case path length),
;;  with the main virtue of small size and simple, easier to understand code.
;;
;;  This code fits in less than 2K of program space, and takes advantage of
;;  that fact at various places to save a few bytes of code, and so it may
;;  not necessarily work as intended if the code size is increased.
;; 
;;  This code does not make use of automatic baud-rate detection, nor does
;;  it use timer1, the low-power, wake-up, analog comparator or A/D
;;  capabilities of the chip.
;;
;;  The interrupt used by this program is timer0, which is intended to create
;;  a periodic (32 us) clock.  Since the timer is reset inside of the ISR, the
;;  foreground routine should disable interrupts for as short a time as
;;  possible (or not at all, if this can be accomplished).
 
;;RenServo modifications/notes
;;5/22/2015 - removed all references to the pic16f688 and cleaned up 16f1825
;;		if statements.  Tested OK.
;;5/23/2015 - removed all AC options.  Removed DC_Build and PWM references(everything 
;;		is DC and PWM.  Tested OK.
;;	    - made minor adjustments to get rid of invalid memory location warnings.
;;5/25/2015 - decided to try calculating timer 1 values and using them for comparators
;;	    - 
;;5/27/2015 - The timer1 values basically work as follows:
 ;;	    -	Each 8 bit Cell Value equates to 15 increments of timer1
 ;;	    -	So the Vixen cell value is multiplied by 15 and added to 1080.  
 ;;	    -	1080 is the value of timer 1 that equals .54ms.  The calculated
 ;;	    -	timer values are then placed in the comparitor(s) to signal an
 ;;	    -	interrupt when the time is reached.
;;6/18/2016 - moved variables from udata block to cblock to make work with msasm.
;;7/24/2016 - Fixed channel order and added ORIGRENSERVO channel order
 ;;	    - Fixed ServoMAX/MIN assignments to correct channel.
 
 LIST R=DEC
 
 ;RENSERVO specific defines  
 ;If you want all outputs to be servos set this to 1
 ;Set to 0 if you want half servos/half dc outputs
    #DEFINE ALLSERVOS 1
 ;If you are using the origional RenServo board(designed by ctmal) which 
 ;requires a different output order change the value below to 1.
    #DEFINE ORIGRENSERVO 0
    ;min/max values change these to change min/max values of servos
    ;note - only Servos 4-7 will be appied if in half servo/half light mode.
    ;programmer note:  move these to EEPROM if remote programming enabled
    ;  Defaults are set for a range of .9ms to 2.1ms
    
    #DEFINE SERVO0MIN	48	;PORTA
    #DEFINE SERVO0MAX	208	
    #DEFINE SERVO1MIN	48	;PORTA
    #DEFINE SERVO1MAX	208
    #DEFINE SERVO2MIN	48	;PORTA
    #DEFINE SERVO2MAX	208
    #DEFINE SERVO3MIN	48	;PORTA
    #DEFINE SERVO3MAX	208
    #DEFINE SERVO4MIN	48	;PORTC
    #DEFINE SERVO4MAX	208
    #DEFINE SERVO5MIN	48	;PORTC
    #DEFINE SERVO5MAX	208
    #DEFINE SERVO6MIN	48	;PORTC
    #DEFINE SERVO6MAX	208
    #DEFINE SERVO7MIN	48	;PORTC
    #DEFINE SERVO7MAX	208

 

;; Configuration for PIC16F1825

   #DEFINE BANK0 0
   #DEFINE BANK1 0x80
   #DEFINE BANK2 0x100
   #DEFINE BANK3 0x180
   #DEFINE BANK4 0x200
   #DEFINE BANK5 0x280
   #DEFINE BANK6 0x300
   #DEFINE BANK7 0x380
   #DEFINE BANK8 0x400
	
    INCLUDE "p16f1825.inc"
	
;; To use the internal clock of the PIC, comment out this line, remove the
;; comments on the next line, and change the clock rate from 18432000 to
;; 32000000.
;;    __CONFIG _CONFIG1, _FOSC_ECH & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_ON & _CLKOUTEN_OFF & _IESO_ON & _FCMEN_OFF
	
    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_ON & _CLKOUTEN_OFF & _IESO_ON & _FCMEN_OFF

    #IFDEF _BORV_HI
    __CONFIG _CONFIG2, _PLLEN_ON & _WRT_OFF & _STVREN_OFF & _BORV_HI & _LVP_OFF
    #ENDIF
	
    #IFDEF _BORV_27
    __CONFIG _CONFIG2, _PLLEN_ON & _WRT_OFF & _STVREN_OFF & _BORV_27 & _LVP_OFF
    #ENDIF
	
    #IFDEF _BORV_25
    __CONFIG _CONFIG2, _PLLEN_ON & _WRT_OFF & _STVREN_OFF & _BORV_25 & _LVP_OFF
    #ENDIF
 CBLOCK 0x020

  ctr_ckt0, ctr_ckt1, ctr_ckt2, ctr_ckt3
  ctr_ckt4, ctr_ckt5, ctr_ckt6, ctr_ckt7

  slot_id_ckt0, slot_id_ckt1, slot_id_ckt2, slot_id_ckt3	
  slot_id_ckt4, slot_id_ckt5, slot_id_ckt6, slot_id_ckt7	

  rx_copycount
  out_map, disable_map, cur_mask

  rx_char, rx_state

  zc_delayline

  timer0_preset

  ctr_timeslot

  our_addr
  rx_discard_count

 ;;RenServo Variables
    timer1ValueH
    timer1ValueL
    cellValue
    servoctrl
    servoPhase
    servoCmask  ;mask for PortC servos
    servoAmask   ;mask for PortA servos
    servoCompleteCtr  ; counter to determine when servos are finished
    servoBanks   ; total number of banks(to populate servoCompleteCtr)
    ;;variables for min/max
    servo0min
    servo0max
    servo1min
    servo1max
    servo2min
    servo2max
    servo3min
    servo3max
    servo4min
    servo4max
    servo5min
    servo5max
    servo6min
    servo6max
    servo7min
    servo7max
    currentmax	;temporary variables used for comparisons
    currentmin
  
  ENDC  
  
 
 #define timer1PresetL	56	;1080 - the value that gets .54ms
 #define timer1PresetH	4
 
 ;;servoctrl bits:
 #define    HAVECELLVALS    0
 

;; assigned numbers:

 #define DEFAULT_LINERATE 50
 #define MAX_LINERATE 70
	
;; derived numbers:

;; initializer for baudrate generator (based on formula from datasheet)
	
 #define BAUD_INIT (CLOCKRATE/BAUDRATE/16 - 1)

;; initializer for cell timer (replaces TIMER_MAGIC0/1).  This assumes 50 Hz,
;; but it will self-adjust after 1/2 AC cycle (better too slow than too fast).

 #define CELL_RATE ((2*DEFAULT_LINERATE) * 256)
 #define CELL_TMR_INIT (CLOCKRATE/4/CELL_RATE)

;; Number used to prevent overly large values from being stored in the
;; timer0_preset variable register.

 #define MIN_TMR1H ((CLOCKRATE/4)/MAX_LINERATE/256/2)
	 	
;; I/O bit definition

 #define TEST_STROBE PORTA,0
;; ---------------------------------------------------------------------------

 org 0
	nop
	goto initialize

 org 4
	;; interrupt routine
	;; Just the timer interrupt for now...
Int
	BANKSEL	TMR0
	btfss	PIR1,CCP1IF
	IF (ALLSERVOS)
	    bra	testCCP2
	ELSE
	    bra	doLights
	ENDIF
	
	bcf	PIR1,CCP1IF	; turn of PORTC servos
	banksel	LATC
	clrf	LATC
	banksel	servoCompleteCtr
	decf	servoCompleteCtr,f	    ;decrease our running bank counter
	
	IF (ALLSERVOS)
testCCP2:
	btfss   PIR2,CCP2IF
	bra	    isr_exit
	bcf	    PIR2,CCP2IF
	banksel LATA
	movlw   b'100000'	;ignore bit 5, it's not a servo(reserved for later use)
	andwf   LATA,f
	banksel servoCompleteCtr
	decf    servoCompleteCtr,f
	ENDIF
	
	IF (!ALLSERVOS)
doLights:
	btfss	INTCON,T0IF
	bra	isr_exit
	bcf	INTCON,T0IF	; reset the interrupt

	movf	timer0_preset,w
	addwf	TMR0,f

	incfsz  ctr_timeslot,f	; decrement slot time counter.
	goto	do_regular

;;
;; ISR code executed if ZC is detected (or counter expires, if DC build)
;;
do_zc:		
	movlw	CTR_LOCKOUT	; dead periods for dimmest possible values
	movwf	ctr_timeslot

turn_off_outputs:
	decf	ctr_timeslot,f	; restore ctr_timeslot in case of jump to here

;;
;; Initialize the ctr_ckt registers for upcoming half-cycle
;;
	movf	slot_id_ckt0,w
	movwf	ctr_ckt0
	
	movf	slot_id_ckt1,w
	movwf	ctr_ckt1
	
	movf	slot_id_ckt2,w
	movwf	ctr_ckt2
	
	movf	slot_id_ckt3,w
	movwf	ctr_ckt3

;; turn outputs off (positive if NEGATIVE_TRUE, ground if POSTIVE_TRUE)

	swapf	disable_map,w	; disable_map tells which outputs to turn off

	rrf	disable_map,w
	andlw	0x7
	btfsc	STATUS,C
	iorlw	0x10
	movwf	PORTA

	goto	isr_exit

;;
;; handle the normal case (next timeslot when no zero-crossing, no timeout).
;;
do_regular:

	clrf	out_map		; map of outputs
			
	incfsz	ctr_ckt0,f
	bsf	out_map,3	; turn off this SSR
	incfsz	ctr_ckt1,f
	bsf	out_map,0
	incfsz	ctr_ckt2,f
	bsf	out_map,1
	incfsz	ctr_ckt3,f
	bsf	out_map,2

   IFDEF OUTPUT_NEGATIVE_TRUE
	swapf	out_map,w
	BANKSEL	LATC
	;andwf	LATC ^ BANK2,f
	swapf	WREG,f
	andlw	0x0F
	addlw	8
	andwf	LATA ^ BANK2,f
   ELSE
	swapf	out_map,w
	xorlw	0xFF
	BANKSEL	LATC
	;iorwf   LATC ^ BANK2,f
	swapf	WREG,f
	andlw	0x0F
	addlw	8
	iorwf	LATA ^ BANK2,f
   ENDIF
   
   ENDIF
isr_exit:
	retfie

;;
;; End of Interrupt Routine, start of Foreground code.
;;

initialize:
;;
;; initialize some registers
;;
	BANKSEL	slot_id_ckt0
	
	IF  ALLSERVOS
	    movlw   128	    ;center the servos
	ELSE
	    movlw   0	    ;set lamps to lowest intensity
	    ;movlw   254    ;testing
	ENDIF
	
	movwf	slot_id_ckt0
	movwf	slot_id_ckt1
	movwf	slot_id_ckt2
	movwf	slot_id_ckt3
	movlw	128	    ; these are always servos
	movwf	slot_id_ckt4
	movwf	slot_id_ckt5
	movwf	slot_id_ckt6
	movwf	slot_id_ckt7
	

	;;set min/max values
	banksel	servo0min
	movlw	SERVO0MIN
	movwf	servo0min
	movlw	SERVO0MAX
	movwf	servo0max
	movlw	SERVO1MIN
	movwf	servo1min
	movlw	SERVO1MAX
	movwf	servo1max
	movlw	SERVO2MIN
	movwf	servo2min
	movlw	SERVO2MAX
	movwf	servo2max
	movlw	SERVO3MIN
	movwf	servo3min
	movlw	SERVO3MAX
	movwf	servo3max
	movlw	SERVO4MIN
	movwf	servo4min
	movlw	SERVO4MAX
	movwf	servo4max
	movlw	SERVO5MIN
	movwf	servo5min
	movlw	SERVO5MAX
	movwf	servo5max
	movlw	SERVO6MIN
	movwf	servo6min
	movlw	SERVO6MAX
	movwf	servo6max
	movlw	SERVO7MIN
	movwf	servo7min
	movlw	SERVO7MAX
	movwf	servo7max
	
	movlb	0	;back to bank 0
	clrf	rx_state
	clrf	ctr_timeslot
	
	movlw LOW (~CELL_TMR_INIT)
	movwf	timer0_preset
	
;; configure the I/O ports

	BANKSEL	CM1CON0
	bcf	CM1CON0 ^ BANK2,C1ON ; disable comparator 1 (register in bank 2)
	movlw	0x00
	movlb	2
	movwf	APFCON0
	movwf	APFCON1
	
	BANKSEL	PORTA
	clrf	out_map
 IFDEF OUTPUT_NEGATIVE_TRUE
	movlw	0xFF
   ELSE
	movlw	0x00
   ENDIF
	movwf	disable_map	; value to turn off outputs at zc time

	movwf	PORTA
	movwf	PORTC
  
	BANKSEL	OSCCON
	movlw	(1<<IRCF3) | (1<<IRCF2) | (1<<IRCF1); | (1<<IRCF0)
	movwf	OSCCON ^ BANK1		; register is in bank 1

	BANKSEL	ANSELA		; Bank 3
	clrf	ANSELA ^ BANK3

	BANKSEL	WPUA
	;movlw	0x20		; turn on weak pullup for unused input (RA5)
	movlw	0x24		; turn on weak pullup for unused inputs(RA5,RA3)
	movwf	WPUA ^ BANK4
	;clrf	WPUA		; turn on weak pullup of PORTA

	BANKSEL	TRISA
	
	;movlw	0x20		; bit 5 is the clock input bit...just in case
	clrf	TRISA ^ BANK1	; h/w ignores bit 3 of this register

	movlw	0xE0		; upper four bits are tri-stated for uart op.
	movwf	TRISC ^ BANK1		; 
	bcf	OPTION_REG ^ BANK1,7	; global weak-pullup enable

	BANKSEL	INLVLA
	clrf	INLVLA ^ BANK7	; TTL Levels
	clrf	INLVLC ^ BANK7
;; Read our address from the EEPROM at the end of this file

;; these lines will cause some assembler warnings, but that is expected and OK

	BANKSEL	EEADR

	BANKSEL	EEADR
	
	movlw	0x00
	movwf	EEADR ^ BANK3
	movwf	EEADRH ^ BANK3
	movwf	EECON1 ^ BANK3

	bsf	EECON1 ^ BANK3,RD
	movf	EEDAT ^ BANK3,w

	BANKSEL	our_addr
	movwf	our_addr

	movlw	0x01		; convert 1-based value from EEPROM into
	subwf	our_addr,f	; 0-based value used in rx routine.


	BANKSEL	BAUDCON
	
;; configure the UART

	clrf	BAUDCON ^ BANK3
	movlw	BAUD_INIT
	movwf	SPBRG ^ BANK3
	clrf	SPBRGH ^ BANK3
		
	movlw	(1<<TXEN) | (1<<BRGH); enable 8-bit transmitter in async mode
				; (x16 baud divisor)
	movwf	TXSTA ^ BANK3
	movlw	(1<<SPEN) | (1<<CREN); enable serial port, continuous 8-bit rx
	movlb	3
	movwf	RCSTA
	
	BANKSEL	T1CON
	
	
;; Set up timer1 - 16 bit counter for our servo comparers
;; 1:4 prescalar
	;movlw	b'00101100'  ; timer is disabled
	movlw	b'00100100'	; timer is disabled
	movwf	T1CON

;; Set up timer0 - generate periodic interrupt at 512 times powerline freq
    IF (!ALLSERVOS)	
	BANKSEL	OPTION_REG
	movlw	0x08		; assign prescaler to WDT (i.e. not used)
	movwf	OPTION_REG ^ BANK1; clear all bits -> timer mode, clk/2
	BANKSEL TMR0
	
	movf	timer0_preset,w
	movwf	TMR0
	movlw	(1 <<GIE) | (1<<T0IE)
	movwf	INTCON		; enable interrupts
    ENDIF
    
;; Set up comparitor 1 - Port C Servo control
    clrf    PIR1
    clrf    PIR2
    banksel CCP1CON	; bank 5
    movlw   b'00001010'
    movwf   CCP1CON
    banksel PIE1
    bsf	    PIE1,CCP1IE	    ; turn on comparitor interrupt
    banksel INTCON
    bsf	    INTCON,PEIE	    ; peripheral interrupts
    bsf	    INTCON,GIE	    ; global interrupts
    movlw   1
    movwf   servoBanks	    ; initialize for this bank
    
;; Set up comparitor 2 - PORT A Servo Control
    IF ALLSERVOS
	banksel	CCP2CON
	movlw	b'00001010'
	movwf	CCP2CON
	banksel	PIE2
	bsf	PIE2,CCP2IE
	banksel	servoBanks
	incf	servoBanks,f	; add this bank to pool
    ENDIF
    
    movlb   0	    ;make sure in bank 0
    
;; Set up Timer 2 - 5ms pulse(starts current phase pulse)
    ; Timer2 Registers: Prescaler=1:16; TMR2 PostScaler=1:10; PR2=250 - Freq = 200.00Hz - Period = 5.00 ms
    movlw   b'01001110'
    movwf   T2CON   ;enabled
    movlw   250
    movwf   PR2
    
    
    
    movlw   4 ;
    movwf   servoPhase
    clrf    servoctrl
    movf    servoBanks,w
    movwf   servoCompleteCtr
    movlb   0	    ;just in case
    
    
    
;; Next two wait loops ensure that we are on the rising edge of ZC when we
;; first enter the mainloop, so that we don't put an overly small value in the
;; timer0_preset register.

mainloop:
	movf	servoCompleteCtr,w	;check to see if current phase is finished
	addlw	0
	btfsc	STATUS,Z
	bra	completePhase
	
	btfsc	PIR1,TMR2IF	;check if we need to start the servo pulse
	bra	startServoPulse
	
	btfss	servoctrl,HAVECELLVALS	;check if we need to set up our cell values
	bra	getServoCells
	
checkRX:
	btfss	PIR1,RCIF	; skip if UART has a rx character to process
	goto	mainloop	; no character available

	bcf	PIR1,RCIF

	BANKSEL	RCREG
	movf	RCREG ^ BANK3,w
	BANKSEL	rx_char
	movwf	rx_char		; and save a working copy
	
	movlw	6		; Rx error mask (framing error, overrun error)
	
	BANKSEL	RCSTA
	andwf	RCSTA,w		; get (masked) rx status
	BANKSEL	PORTA	
	btfss	STATUS,Z
	goto	rx_error	; goto error routine

	movf	rx_char,w	; retrieve the character
	addlw	0x82		; test for sync character
	btfss	STATUS,Z	; skip if sync character
	goto	rx_not_sync	; jump to code for processing non-sync
;; 
;; process sync character
;;
	bcf	rx_state,3	; exit the discarding state
	bcf	rx_state,2	; start out just echoing data
	bsf	rx_state,1
rx_echo:	
	movf	rx_char,w	; echo most recent character to transmitter

	BANKSEL	TXREG
	movwf	TXREG ^ BANK3
	BANKSEL	PORTA

	goto	mainloop

rx_error:
	BANKSEL	RCSTA
	bcf	RCSTA,CREN
	bsf	RCSTA,CREN
	BANKSEL	PORTA

	bsf	rx_state,3	; start discarding
	
	goto	mainloop

;; 
;; process non-sync character
;;
;; rx_state:
;; xxxx1xxx -> discarding data (startup, just after rx error)
;; xxxx01xx -> echoing data (count was > 0 or after end of local data)
;; xxxx001x -> expecting command/address byte (just after SYNC)
;; xxxx0001 -> process character after an ESC character
;; xxxx0000 -> process short-form data character

rx_not_sync:
	addlw	1		; test for PAD character - discard if yes
	
	btfsc	STATUS,Z
	goto	mainloop
	
	btfsc	rx_state,3	; skip if not discard state
	goto	mainloop
	btfsc	rx_state,2	; skip if non-echo state
	goto	rx_echo		; pure echo
;;
;; handle non-echo cases
;;
	btfss	rx_state,1	; skip if first character after SYNC
	goto	decode_packet_body
decode_cmd_byte:		; first character after the SYNC

	btfss	rx_char,7
	goto	rx_echo		; long form packet, just echo it.

	movf	our_addr,w
	subwf	rx_char,f	; incoming address - 'our_addr'

	btfsc	rx_char,7	; bit 7 set after the subtract?  If so, data is
	goto	rx_echo		; all for downstream controllers.

	movlw	0x81
	addwf	rx_char,w
	movwf	rx_discard_count
	
	movlw	0x80
	movwf	rx_char
	clrf	rx_state	; next state is normal body

	movlw	1
	movwf	cur_mask
	clrf	rx_copycount
	
	goto	rx_echo		; get next character (and echo the '0x80').
;;
;; done initializing packet, now handle the data
;;
decode_packet_body:	
	bsf	rx_state,2	; just in case packet is not for us.

	btfss	rx_state,0	; skip if previous char was ESC
	goto	previous_not_ESC
	movlw	0x4E
	addwf	rx_char,f	; decode the ESC charactor
	goto	after_ESC_decoding
previous_not_ESC:	
	
	bsf	rx_state,0	; just in case current char is ESC
	movlw	0x7F
	xorwf	rx_char,w	; check for ESC
	btfsc	STATUS,Z	; skip if not ESC
	goto	mainloop	; go and discard the ESC
after_ESC_decoding:
	clrf	rx_state	; normal data state

	movf	rx_discard_count,f
	btfss	STATUS,Z
	goto	rx_upfront_discard

	movlw	slot_id_ckt0
	addwf	rx_copycount,w

	movwf	FSR0L
	clrf	FSR0H
	
	movf	rx_char,w
	movwf	INDF0
	
	addlw	1		; check for 255
	movf	disable_map,w	; update the mask of full-brightness channels
	iorwf	cur_mask,w
    IFDEF OUTPUT_NEGATIVE_TRUE
	btfsc	STATUS,C
	xorwf	cur_mask,w	; clr bit in cur_mask if channel brightness = 0
    ELSE
	btfss	STATUS,C	
	xorwf	cur_mask,w	; set bit in cur_mask if channel brightness = 0
    ENDIF
	movwf	disable_map

	addlw	0		; clear the carry
	rlf	cur_mask,f	; update mask for current channel


	incf	rx_copycount,f	; count of bytes copied into memory
	btfsc	rx_copycount,3  ; 8 bytes copied into memory?
	bsf	rx_state,2	; if yes, start the 'echo' rx_state
	goto	mainloop	; no echo !

rx_upfront_discard:
	incf	rx_copycount,f

	btfsc	rx_copycount,3
	incf	rx_discard_count,f

	bcf	rx_copycount,3

	goto	mainloop
	
completePhase:
	bcf	T1CON,TMR1ON
	movf	servoBanks,w
	movwf	servoCompleteCtr
	clrf	servoctrl
	decfsz	servoPhase,f
	bra	mainloop
	movlw	4
	movwf	servoPhase  ;restore to 1st servo phase
	bra	mainloop
	
;;enter with cellValue populated
	
startServoPulse:
    bcf	    PIR1,TMR2IF
    banksel TMR1H
    clrf    TMR1H	;clear the timer
    clrf    TMR1L
    bsf	    T1CON,TMR1ON	;turn timer1 on
    movf    servoCmask,w	;turn on portc servo
    banksel LATC
    xorwf   LATC,f
    banksel servoAmask
    IF ALLSERVOS
    movf    servoAmask,w
    banksel LATA
    xorwf   LATA,f
    banksel servoAmask
    ENDIF
    bra	    mainloop
	
getServoCells:
    
    call    getServoCmask
    movwf   servoCmask  
    IF (ALLSERVOS)
	movwf	servoAmask
	btfsc	servoAmask,3	;check for special case
	lslf	servoAmask,f
    ENDIF
    call    getCcellValue;
    call    convertCellToTimer
    movf    timer1ValueH,w  ; put values in correct comparitor
    movlb   5
    movwf   CCPR1H
    movlb   0
    movf    timer1ValueL,w
    movlb   5
    movwf   CCPR1L
    movlb   0
    
    IF (ALLSERVOS)
	call    getAcellValue;
	;movwf   cellValue;
	call    convertCellToTimer
	movf    timer1ValueH,w  ; put values in correct comparitor
	movlb   5
	movwf   CCPR2H
	movlb   0
	movf    timer1ValueL,w
	movlb   5
	movwf   CCPR2L
	movlb   0
    ENDIF
    bsf	    servoctrl,0
    bra	    mainloop;
    
    
getServoCmask:
    decf    servoPhase,w
    brw
    retlw   b'00001'
    retlw   b'00010'
    retlw   b'00100'
    retlw   b'01000'
    
getCcellValue:
    decf    servoPhase,w
    lslf    WREG,w ; multiply x2
    lslf    WREG,w  ;x4
    lslf    WREG,w  ;x8
    brw
    movf   slot_id_ckt4,w
    movwf   cellValue		;put our value in cellvalue
    movf    servo4min,w		
    movwf   currentmin
    movf    servo4max,w
    movwf   currentmax
    call    calcMinMax
    return
    movf   slot_id_ckt5,w
    movwf   cellValue		;put our value in cellvalue
    movf    servo5min,w		
    movwf   currentmin
    movf    servo5max,w
    movwf   currentmax
    call    calcMinMax
    return
    movf   slot_id_ckt6,w
    movwf   cellValue		;put our value in cellvalue
    movf    servo6min,w		
    movwf   currentmin
    movf    servo6max,w
    movwf   currentmax
    call    calcMinMax
    return
    movf   slot_id_ckt7,w	;servo7
    movwf   cellValue		;put our value in cellvalue
    movf    servo7min,w		
    movwf   currentmin
    movf    servo7max,w
    movwf   currentmax
    call    calcMinMax
    return
    
getAcellValue:
    decf    servoPhase,w
    lslf    WREG,w ; multiply x2
    lslf    WREG,w  ;x4
    lslf    WREG,w  ;x8
    brw
    IF (ORIGRENSERVO)
	movf	slot_id_ckt0,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo0min,w		
	movwf   currentmin
	movf    servo0max,w
	movwf   currentmax
	call    calcMinMax
	return 
	movf	slot_id_ckt1,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo1min,w		
	movwf   currentmin
	movf    servo1max,w
	movwf   currentmax
	call    calcMinMax
	return
	movf	slot_id_ckt2,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo2min,w		
	movwf   currentmin
	movf    servo2max,w
	movwf   currentmax
	call    calcMinMax
	return
	movf	slot_id_ckt3,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo3min,w		
	movwf   currentmin
	movf    servo3max,w
	movwf   currentmax
	call    calcMinMax
	return
    ELSE
	movf	slot_id_ckt1,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo1min,w		
	movwf   currentmin
	movf    servo1max,w
	movwf   currentmax
	call    calcMinMax
	return
	movf	slot_id_ckt2,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo2min,w		
	movwf   currentmin
	movf    servo2max,w
	movwf   currentmax
	call    calcMinMax
	return
	movf	slot_id_ckt3,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo3min,w		
	movwf   currentmin
	movf    servo3max,w
	movwf   currentmax
	call    calcMinMax
	return
	movf	slot_id_ckt0,w
	movwf   cellValue		;put our value in cellvalue
	movf    servo0min,w		
	movwf   currentmin
	movf    servo0max,w
	movwf   currentmax
	call    calcMinMax
	return
    ENDIF
calcMinMax:
    ;enter with cellvalue,currentmin,currentmax assigned
    ;returns correct value in cellvalue
    movf    currentmin,w	;test min value
    subwf   cellValue,w
    btfsc   STATUS,C		
    bra	    calcMax	    ;value is greater than min, need to test max
    movf    currentmin,w    ;value is lower than min, make min the cellvalue
    movwf   cellValue;
    return	    ;no need to check max since it violated min
    
calcMax:
    movf    currentmax,w	;test max value
    subwf   cellValue,w
    btfss   STATUS,C
    return		; value is less than max value
    movf    currentmax,w
    movwf   cellValue;
    return
convertCellToTimer:
    clrf timer1ValueH;

    movf	cellValue,w  	;place cell value in timer1ValueL for calculating
    movwf	timer1ValueL

    ;;enter with Vixen value in timerValueL
    lslf    timer1ValueL,f	;multiply times 2
    rlf	    timer1ValueH,f	;rotote carry into high bit
    lslf    timer1ValueL,f	; cellValue x 4
    rlf	    timer1ValueH,f
    lslf    timer1ValueL,f	; cellValue x 8
    rlf	    timer1ValueH,f
    lslf    timer1ValueL,f	; cellValue x 16
    rlf	    timer1ValueH,f

    movf	cellValue,w	; cellValue x 15
    subwf	timer1ValueL,f
    btfss	STATUS,C
    decf	timer1ValueH,f

    ;add timer info to presets
    movlw	timer1PresetL
    addwf	timer1ValueL,f
    btfsc	STATUS,C
    incf	timer1ValueH,f
    movlw	timer1PresetH
    addwf	timer1ValueH,f
    return

;; data stored in the eeprom.  First byte is MSB of our address (0/1), second
;; byte is the LSB of our address (0-255).

  ORG 0xF000

   de 1+START_ADDR

  END
